#
# readrc.rb
#
#   mailest resource parser
#
#   Copyright(c) 1999 Minero Aoki
#   aamine@dp.u-netsurf.ne.jp
#

module EST

class ConfigError < StandardError ; end

class RcParser

  def RcParser.parse( fname )
    p = new( fname )
    p.parse
  end

  def initialize( fname )
    @filename = fname
  end

  def parse
    ret = nil

    File.open( @filename ) do |f|
      ret = parse_conffile( f )
#puts ret.type
    end
#puts ret.type

    return ret
  end


  private


  def parse_conffile( f )
    @lineno = 0
    return parse_block( f, true )
  end


  def parse_block( f, toplevel = false )
    hash = {}

    f.each do |line|
#puts line
      @lineno += 1

      if /\#/o === line then
        line = $`
      end
      line.strip!

      case line
      when /\A([\w\-]+)\s+\"([\w\- ]+)\"\s*\{\z/o
#puts 'begin block'
        entry = $1
        name = $2

        unless ret = hash[ entry ] then
          hash[ entry ] = ret = {}
        end

        if ret[ name ] then
          configerror! "entry '#{name}' exist twice"
        end
        ret[ name ] = parse_block( f )

      when /\A([\w\-]+)\s*\{\z/o, /\A\"([\w\-]+)\"\s*\{\z/o
#puts 'begin global block'
        name = $1
        if hash[ name ] then
          configerror! "config entry '#{name}' exist twice"
        end
        hash[ name ] = parse_block( f )

      when /\A\}\z/o
#puts 'end block'
#puts hash.type
        return hash
      
      when ''
#puts 'void line'
        ;
      
      else
#puts 'value line'
        key, val = line.split(/[=:]/o, 2)
        key.strip!
        val.strip!

        hash[ key ] = val
      end
    end

    unless toplevel then
      configerror! "number of '{' is more than the one of '}'"
    end
    return hash
  end



  def configerror!( mes )
    raise ConfigError,
      "error in config file #{@fname}(line #{@lineno}): " + mes
  end

end


class RcReader

  def RcReader.read( fn )
    ent = RcParser.parse( fn )
    reader = new
    reader.configure ent
    reader.configure_if ent['interface'][ Interface ]

    return ent['first_profile']
  end

  def configure( entries )
    setup_mailbox  entries['mailbox']
    setup_smtpsrv  entries['smtp-server']
    setup_popsrv   entries['pop-server']
    setup_composer entries['composer']
    setup_refile   entries['refile']
    setup_profile  entries['profile']
  end

  # abstract configure_if


  def setup_smtpsrv( config )
    config.each do |name, rc|
      CORE.add_sender name, rc
    end
  end

  def setup_popsrv( config )
    config.each do |name, rc|
      CORE.add_popper name, rc
    end
  end

  def setup_mailbox( config )
    config.each do |name, rc|
      CORE.mailbox.add_mbox name, rc
    end
  end

  def setup_composer( config )
    config.each do |name, rc|
      CORE.add_composer name, rc
    end
  end

  def setup_refile( config )
    return unless config

    config.each do |name, rc|
      arr = []
      rc.each do |num, str|
        # "str" is like [ header, refile_regex, dest_mbox ]
        arr[ num.to_i ] = eval( str )
      end
      arr.compact!
      CORE.add_refile_rule name, arr
    end
  end

  def setup_profile( config )
    config.each do |name, rc|
      rc['pop-server'] = rc['pop-server'].split(',').filter{|i| i.strip! }
      CORE.add_profile name, rc
    end
  end

  def confirm_config( profnm, profiles )
    ent = profiles[ profnm ]
    unless ent['mailbox'] and
           ent['smtp-server'] and
           ent['pop-server'] and
           ent['composer']
      CORE.warn "EST Warning: profile #{profnm} not exist: delete entry"
      profiles.delete profnm
    end
  end

end

end   # module EST
