
require 'racc/parser'
#
# mailp.rb
#
#   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#


class ScanError < StandardError; end

begin
  require 'tmail/mails.so'
rescue LoadError
  require 'tmail/mails.rb'
end


module TMail


class Mailp < Racc::Parser

  MAILP_DEBUG = false

  def initialize
    self.debug = MAILP_DEBUG
  end

  def debug=( flag )
    @yydebug = flag && DEBUG_PARSER
    @scanner_debug = flag
  end

  def debug
    @yydebug
  end


  def Mailp.parse( str, obj, ident )
    new.parse( str, obj, ident )
  end


  def parse( str, obj, ident )
    return if /\A\s+\z/ === str

    @field = obj

    case obj
    when MsgidH
      msgid_parse str
    when RefH
      refs_parse str
    else
      unless ident then
        ident = obj.type.name.split('::')[-1].to_s
        cmt = []
        obj.comments.replace cmt
      else
        cmt = nil
      end

      @scanner = MailScanner.new( str, ident, cmt )
      @scanner.debug = @scanner_debug
      @first = [ ident.intern, ident ]
      @pass_array = [nil, nil]

      do_parse
    end
  end


  private


  def next_token
    if @first then
      ret = @first
      @first = nil
      ret
    else
      @scanner.scan @pass_array
    end
  end
  
  def on_error( tok, val, vstack )
    raise ParseError,
      "\nparse error in '#{@field.name}' header, on token #{val.inspect}"
  end


  def refs_parse( str )
    arr = []

    while ::TMail::MSGID === str do
      p   = $`
      m   = $&
      str = $'

      p.strip!
      arr.push p
      arr.push m
    end
    str.strip!
    arr.push str unless str.empty?

    @field.refs.replace arr
  end

  def msgid_parse( str )
    if ::TMail::MSGID === str then
      @field.msgid = $&
    else
      raise ParseError, "wrong Message-ID format: #{str}"
    end
  end


##### racc version 0.10.8 generates ###

LR_reduce_table = [
0, 0, :racc_error,
2, 42, :_reduce_1,
2, 42, :_reduce_none,
2, 42, :_reduce_none,
2, 42, :_reduce_4,
2, 42, :_reduce_5,
2, 42, :_reduce_6,
2, 42, :_reduce_7,
2, 42, :_reduce_8,
2, 42, :_reduce_9,
2, 42, :_reduce_10,
2, 42, :_reduce_none,
2, 42, :_reduce_none,
2, 42, :_reduce_none,
2, 42, :_reduce_none,
2, 42, :_reduce_none,
2, 42, :_reduce_16,
2, 42, :_reduce_17,
6, 41, :_reduce_18,
0, 57, :_reduce_none,
2, 57, :_reduce_none,
3, 58, :_reduce_21,
5, 58, :_reduce_22,
1, 59, :_reduce_23,
7, 43, :_reduce_24,
0, 60, :_reduce_none,
2, 60, :_reduce_26,
4, 60, :_reduce_27,
3, 60, :_reduce_28,
0, 61, :_reduce_none,
2, 61, :_reduce_30,
0, 62, :_reduce_none,
2, 62, :_reduce_32,
0, 63, :_reduce_33,
3, 63, :_reduce_34,
0, 64, :_reduce_none,
2, 64, :_reduce_36,
2, 64, :_reduce_37,
0, 65, :_reduce_none,
2, 65, :_reduce_39,
0, 66, :_reduce_none,
2, 66, :_reduce_41,
2, 44, :_reduce_none,
1, 44, :_reduce_43,
1, 45, :_reduce_44,
3, 45, :_reduce_45,
1, 46, :_reduce_none,
1, 46, :_reduce_none,
1, 47, :_reduce_48,
3, 47, :_reduce_49,
1, 48, :_reduce_none,
1, 48, :_reduce_none,
2, 48, :_reduce_52,
4, 69, :_reduce_53,
4, 68, :_reduce_54,
3, 68, :_reduce_55,
2, 71, :_reduce_none,
2, 72, :_reduce_57,
4, 72, :_reduce_58,
3, 56, :_reduce_59,
1, 56, :_reduce_60,
1, 73, :_reduce_61,
3, 73, :_reduce_62,
1, 67, :_reduce_63,
3, 67, :_reduce_64,
1, 75, :_reduce_none,
1, 75, :_reduce_none,
1, 75, :_reduce_none,
3, 49, :_reduce_68,
1, 70, :_reduce_none,
2, 70, :_reduce_70,
1, 74, :_reduce_none,
1, 74, :_reduce_none,
1, 74, :_reduce_none,
1, 50, :_reduce_none,
3, 50, :_reduce_none,
1, 51, :_reduce_76,
2, 51, :_reduce_77,
3, 52, :_reduce_78,
4, 53, :_reduce_79,
2, 53, :_reduce_80,
0, 77, :_reduce_none,
5, 77, :_reduce_82,
1, 78, :_reduce_none,
1, 78, :_reduce_none,
1, 54, :_reduce_85,
2, 55, :_reduce_86,
0, 79, :_reduce_none,
3, 79, :_reduce_none,
0, 80, :_reduce_none,
3, 80, :_reduce_90,
1, 76, :_reduce_none,
1, 76, :_reduce_none,
1, 76, :_reduce_none,
1, 76, :_reduce_none,
1, 76, :_reduce_none,
1, 76, :_reduce_none,
1, 76, :_reduce_none ]

LR_reduce_n = 98

LR_shift_n = 164

LR_action_table = [
     5,     1,     3,     2,    10,     4,    17,     3,     8,     6,
    15,     5,     6,     9,    13,     8,    19,     7,     4,    10,
     2,    12,    12,    14,    18,    13,     9,    16,    16,    15,
     7,    18,    14,    17,    -1,   -98,    21,    21,    28,    20,
    24,    23,    31,    24,    33,    28,    27,    27,    30,    30,
    36,    38,    20,    37,    29,    36,    -1,   -98,    24,    39,
    -1,   -25,    37,    42,    -1,   -98,    11,    44,    -1,   -98,
    37,    45,    -1,   -98,    21,    21,    28,    20,    24,    23,
    31,    24,    33,    28,    27,    27,    30,    30,    36,    38,
    20,    37,    29,    36,    -1,   -98,    21,    21,    28,    20,
    24,    23,    31,    24,    27,    27,    30,    30,    36,    38,
    20,    37,    29,    36,    -1,   -98,    21,    21,    28,    20,
    24,    23,    31,    24,    27,    27,    30,    30,    36,    38,
    20,    37,    29,    36,    -1,   -98,    21,    21,    28,    20,
    24,    23,    31,    24,    33,    28,    27,    27,    30,    30,
    36,    38,    20,    37,    29,    36,    -1,   -98,    33,    54,
    -1,   -98,     0,    57,    -1,   -98,    21,    59,    20,   -19,
    -1,   -98,    21,    21,    28,    20,    24,    23,    31,    24,
    33,    28,    27,    27,    30,    30,    36,    38,    20,    37,
    29,    36,    -1,   -98,    21,    21,    28,    20,    24,    23,
    31,    24,    27,    27,    30,    30,    36,    38,    20,    37,
    29,    36,    -1,   -98,    37,    65,    -1,   -98,    33,    68,
    -1,   -98,    20,    70,    -1,   -98,    21,    21,    28,    20,
    24,    23,    31,    24,    33,    28,    27,    27,    30,    30,
    36,    38,    20,    37,    29,    36,    -1,   -98,    35,    73,
    25,    74,    -1,   -60,    -1,   -94,    -1,   -91,    21,    21,
    28,    20,    23,    75,    24,    23,    31,    24,    33,    28,
    27,    27,    30,    30,    36,    38,    20,    37,    29,    36,
    -1,   -98,    -1,   -92,    -1,   -97,    22,   -61,    35,   -61,
    32,   -61,    34,   -61,     0,   -61,    25,   -61,    -1,   -69,
    22,    78,     0,    -4,    -1,   -98,    -1,   -93,    21,    21,
    28,    20,    24,    23,    31,    24,    27,    27,    30,    30,
    25,    80,    36,    38,    20,    37,    29,    36,    -1,   -98,
    -1,   -47,    -1,   -96,    -1,   -51,    -1,   -46,    -1,   -71,
    -1,   -50,    -1,   -44,    -1,   -95,    -1,   -73,    -1,   -72,
    21,    21,    28,    20,    24,    23,    31,    24,    27,    27,
    30,    30,    26,    85,    20,    87,    29,    36,    -1,   -98,
    27,    89,    -1,   -29,    -1,    -2,    -1,   -87,    -1,   -15,
    -1,    -9,    38,    92,    -1,   -81,    -1,   -13,    21,    21,
    28,    20,    24,    23,    31,    24,    33,    28,    27,    27,
    30,    30,    36,    38,    20,    37,    29,    36,    -1,   -98,
    -1,    -7,    -1,   -61,    -1,   -17,    21,    21,    28,    20,
    24,    23,    31,    24,    27,    27,    30,    30,     0,   -76,
    36,    38,    20,    37,    29,    36,    -1,   -98,    -1,   -11,
    -1,    -5,    21,    21,    28,    20,    24,    23,    31,    24,
    27,    27,    34,    94,    30,    30,    25,    80,    36,    38,
    20,    37,    29,    36,    -1,   -98,    -1,   -43,    -1,    -3,
     0,    95,    -1,   -98,    -1,    -1,    22,    96,    -1,   -98,
    20,    97,    -1,   -98,    -1,   -16,    21,    21,    28,    20,
    24,    23,    31,    24,    27,    27,    30,    30,    36,    38,
    20,    37,    29,    36,    -1,   -74,    -1,   -69,    22,    98,
     0,   -10,    -1,   -98,    -1,   -85,    -1,   -14,    -1,    -8,
    21,    21,    28,    20,    24,    23,    31,    24,    27,    27,
    30,    30,    36,    38,    20,    37,    29,    36,    -1,   -98,
    -1,   -12,    35,   100,    -1,   -98,    22,   101,     0,    -6,
    -1,   -98,    -1,   -48,    21,    21,    28,    20,    24,    23,
    31,    24,    27,    27,    30,    30,    36,    38,    20,    37,
    29,    36,    -1,   -98,    21,    21,    28,    20,    24,    23,
    31,    24,    27,    27,    30,    30,    26,    85,    20,    87,
    29,    36,    -1,   -98,    21,    21,    28,    20,    24,    23,
    31,    24,    33,    28,    27,    27,    30,    30,    36,    38,
    20,    37,    29,    36,    -1,   -98,    -1,   -70,    -1,   -52,
    21,    21,    28,    20,    24,    23,    31,    24,    33,    28,
    27,    27,    30,    30,    36,    38,    20,    37,    29,    36,
    -1,   -98,    22,   106,    23,   107,    -1,   -98,    21,    21,
    28,    20,    24,    23,    31,    24,    27,    27,    30,    30,
    26,    85,    20,    87,    29,    36,    -1,   -98,    34,   109,
    -1,   -98,    21,    21,    28,    20,    24,    23,    31,    24,
    27,    27,    30,    30,    36,    38,    20,    37,    29,    36,
    -1,   -98,    -1,   -63,    35,   111,    26,   112,    25,   113,
    -1,   -26,    -1,   -66,    -1,   -65,    -1,   -67,    28,   115,
    -1,   -31,    21,    21,    28,    20,    24,    23,    31,    24,
    27,    27,    30,    30,    26,    85,    20,    87,    29,    36,
    -1,   -98,    32,   117,     0,   -86,    -1,   -98,    32,   118,
     0,   -80,    -1,   -98,    37,   119,    -1,   -98,    -1,   -77,
    -1,   -42,    -1,   164,    -1,   -20,    21,   120,    -1,   -98,
    21,    21,    28,    20,    24,    23,    31,    24,    27,    27,
    30,    30,    36,    38,    20,    37,    29,    36,    -1,   -98,
    34,   122,    -1,   -98,    20,   123,    -1,   -98,    21,    21,
    28,    20,    24,    23,    31,    24,    33,    28,    27,    27,
    30,    30,    36,    38,    20,    37,    29,    36,    -1,   -98,
    -1,   -62,    35,   111,    -1,   -59,    22,   101,    32,   125,
    -1,   -98,    -1,   -45,    25,   126,    -1,   -98,    -1,   -56,
    35,   111,    -1,   -57,    -1,   -55,    34,   127,    -1,   -98,
    21,    21,    28,    20,    24,    23,    31,    24,    27,    27,
    30,    30,    26,    85,    20,    87,    29,    36,    -1,   -98,
    -1,   -28,    21,    21,    28,    20,    24,    23,    31,    24,
    27,    27,    30,    30,    26,    85,    20,    87,    29,    36,
    -1,   -98,    -1,   -33,    21,   131,    -1,   -98,    35,   111,
    -1,   -30,    37,   132,    -1,   -89,    37,   134,    -1,   -98,
    -1,   -81,    20,   136,    -1,   -98,    21,    21,    28,    20,
    24,    23,    31,    24,    27,    27,    30,    30,    36,    38,
    20,    37,    29,    36,    -1,   -75,    -1,   -68,    -1,   -78,
    -1,   -49,    -1,   -53,    21,    21,    28,    20,    24,    23,
    31,    24,    27,    27,    30,    30,    26,    85,    20,    87,
    29,    36,    -1,   -98,    -1,   -54,    -1,   -64,    35,   111,
    -1,   -27,    30,   138,    29,   139,    -1,   -35,    -1,   -32,
    39,   141,    -1,   -98,    -1,   -88,    39,   142,    -1,   -98,
    32,   118,     0,   -79,    -1,   -98,    20,   144,    -1,   -98,
    35,   111,    -1,   -58,    21,   145,    33,    68,    -1,   -98,
    21,   147,    -1,   -98,    31,   148,    -1,   -38,    37,   150,
    36,   152,    -1,   -98,    37,   150,    36,   152,    -1,   -98,
    21,   154,    -1,   -98,    23,   156,    -1,   -98,    -1,   -37,
    -1,   -36,    -1,   -34,    21,    21,    28,    20,    24,    23,
    31,    24,    33,    28,    27,    27,    30,    30,    36,    38,
    20,    37,    29,    36,    -1,   -98,    32,   159,     0,   -40,
    -1,   -98,    -1,   -83,    -1,   -90,    -1,   -84,    -1,   -82,
    -1,   -23,    -1,   -18,    20,   160,    -1,   -98,    -1,   -39,
    -1,   -24,    21,    59,    20,   -19,    -1,   -98,    21,   -21,
    23,   162,    -1,   -98,    -1,   -41,    20,   163,    -1,   -98,
    -1,   -22 ]

LR_action_table_ptr = [
     0,    36,    58,    62,    66,    70,    74,    96,   116,   136,
   158,   162,   166,   172,   194,   214,   218,   222,   226,   248,
   254,   256,   258,   282,   284,   286,   300,   306,   308,   330,
   332,   334,   336,   338,   340,   342,   344,   346,   348,   350,
   370,   374,   376,   378,   380,   382,   386,   388,   410,   412,
   414,   416,   438,   440,   442,   466,   468,   470,   474,   476,
   480,   484,   486,   506,   508,   514,   516,   518,   520,   540,
   542,   546,   552,   554,   574,   594,   616,   618,   620,   642,
   648,   668,   672,   692,   694,   702,   704,   706,   708,   712,
   732,   738,   744,   748,   750,   752,   754,   756,   760,   780,
   784,   788,   810,   812,   816,   822,   824,   828,   830,   834,
   836,   840,   860,   862,   882,   884,   888,   892,   896,   900,
   902,   906,   926,   928,   930,   932,   934,   954,   956,   958,
   962,   968,   970,   974,   976,   980,   986,   990,   994,  1000,
  1004,  1008,  1014,  1020,  1024,  1028,  1030,  1032,  1034,  1056,
  1062,  1064,  1066,  1068,  1070,  1072,  1074,  1078,  1080,  1082,
  1088,  1094,  1096,  1100 ]


LR_goto_table = [
    42,    11,    73,    19,    70,    22,    74,    25,    45,    26,
    69,    29,    68,    31,    48,    32,    76,    33,    56,    34,
    46,    35,    60,    40,    43,    41,    55,    43,    53,    46,
    73,    19,    70,    47,    74,    25,    68,    31,    48,    48,
    76,    33,    56,    34,    73,    19,    74,    49,    76,    33,
    56,    50,    74,    51,    76,    33,    51,    52,    73,    19,
    70,    22,    74,    25,    69,    29,    68,    31,    48,    32,
    76,    33,    56,    34,    46,    53,    68,    55,    44,    56,
    41,    58,    57,    60,    73,    19,    70,    47,    74,    25,
    68,    31,    48,    61,    76,    33,    56,    34,    70,    62,
    74,    63,    76,    33,    50,    64,    54,    66,    49,    67,
    52,    69,    73,    19,    70,    47,    47,    71,    74,    25,
    68,    31,    48,    72,    76,    33,    56,    34,    74,    76,
    68,    77,    76,    33,    73,    19,    74,    49,    72,    79,
    76,    33,    56,    81,    71,    82,    75,    83,    67,    84,
    76,    86,    61,    88,    79,    90,    77,    91,    74,    76,
    68,    77,    76,    33,    74,    93,    76,    33,    73,    19,
    74,    49,    72,    79,    76,    33,    56,    81,    71,    82,
    74,    76,    76,    33,    73,    19,    74,    49,    76,    33,
    56,    99,    74,   102,    76,    33,    75,    83,    67,   103,
    76,    86,    73,    19,    70,    47,    47,   104,    74,    25,
    68,    31,    48,    72,    76,    33,    56,    34,    73,    19,
    70,    22,    74,    25,    69,    29,    68,    31,    48,    32,
    76,    33,    56,    34,    46,   105,    75,    83,    67,   108,
    76,    86,    73,    19,    74,    49,    76,    33,    56,   110,
    62,   114,    75,    83,    67,   116,    76,    86,    70,   121,
    74,    63,    76,    33,    73,    19,    70,    47,    74,    25,
    68,    31,    48,   124,    76,    33,    56,    34,    75,   128,
    76,    86,    75,    83,    67,   129,    76,    86,    63,   130,
    80,   133,    77,   135,    74,    76,    76,    33,    75,    83,
    67,   137,    76,    86,    64,   140,    58,   143,    49,   146,
    65,   149,    78,   151,    78,   153,    59,   155,    73,    19,
    70,    22,    74,    25,    69,    29,    68,    31,    48,    32,
    76,    33,    56,    34,    46,   157,    66,   158,    41,   161,
    57,    60,    -1,    -1 ]

LR_goto_table_ptr = [
     0,     2,    22,    26,    -1,    28,    30,    44,    52,    58,
    76,    -1,    80,    84,    98,   106,   108,   110,   112,    -1,
    -1,    -1,   128,    -1,    -1,    -1,    -1,    -1,   134,    -1,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   146,
   152,    -1,   154,    -1,    -1,   156,    -1,   158,    -1,    -1,
    -1,   164,    -1,    -1,   168,    -1,    -1,    -1,    -1,    -1,
    -1,    -1,   180,    -1,    -1,    -1,    -1,    -1,   184,    -1,
    -1,    -1,    -1,   192,   196,   202,    -1,    -1,   218,    -1,
   236,    -1,   242,    -1,    -1,    -1,    -1,    -1,   250,   252,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   258,    -1,
    -1,   264,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
    -1,   278,    -1,   282,   288,    -1,    -1,   290,    -1,   292,
    -1,   294,    -1,    -1,    -1,    -1,   298,    -1,    -1,    -1,
   304,    -1,    -1,    -1,    -1,    -1,   306,    -1,   308,    -1,
   310,   312,   314,   316,    -1,    -1,    -1,    -1,   318,   336,
    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,   338,
    -1,    -1,    -1,    -1 ]


LR_token_table = {
 false => 0,
 Object.new => 1,
 :DateH => 2,
 :RecvH => 3,
 :RetpathH => 4,
 :MaddrH => 5,
 :SaddrH => 6,
 :MmboxH => 7,
 :SmboxH => 8,
 :MsgidH => 9,
 :RefH => 10,
 :refs => 11,
 :KeyH => 12,
 :EncH => 13,
 :VersionH => 14,
 :CTypeH => 15,
 :CEncodingH => 16,
 :CDispositionH => 17,
 :Mbox => 18,
 :Spec => 19,
 :DIGIT => 20,
 :ATOM => 21,
 "," => 22,
 ":" => 23,
 :FROM => 24,
 "@" => 25,
 :DOMLIT => 26,
 :BY => 27,
 :VIA => 28,
 :WITH => 29,
 :ID => 30,
 :FOR => 31,
 ";" => 32,
 "<" => 33,
 ">" => 34,
 "." => 35,
 :QUOTED => 36,
 :TOKEN => 37,
 "/" => 38,
 "=" => 39 }

DEBUG_PARSER = false

##### racc system variables end #####

 # reduce 0 omitted

 def _reduce_1( val, _values, result )
 @field.date  = val[1]
  result
 end

 # reduce 2 omitted

 # reduce 3 omitted

 def _reduce_4( val, _values, result )
 @field.addrs.replace val[1]
  result
 end

 def _reduce_5( val, _values, result )
 @field.addr  = val[1]
  result
 end

 def _reduce_6( val, _values, result )
 @field.addrs.replace val[1]
  result
 end

 def _reduce_7( val, _values, result )
 @field.addr  = val[1]
  result
 end

 def _reduce_8( val, _values, result )
 @field.msgid = val[1]
  result
 end

 def _reduce_9( val, _values, result )
 @field.refs.replace val[1]
  result
 end

 def _reduce_10( val, _values, result )
 @field.keys.replace val[1]
  result
 end

 # reduce 11 omitted

 # reduce 12 omitted

 # reduce 13 omitted

 # reduce 14 omitted

 # reduce 15 omitted

 def _reduce_16( val, _values, result )
        mb = val[1]
        @field.phrase = mb.phrase
        @field.setroute mb.route
        @field.local  = mb.local
        @field.domain = mb.domain
  result
 end

 def _reduce_17( val, _values, result )
        mb = val[1]
        @field.local  = mb.local
        @field.domain = mb.domain
  result
 end

 def _reduce_18( val, _values, result )
      t = Time.gm( val[3].to_i, val[2], val[1].to_i, 0, 0, 0 )
      result = (t + val[4] - val[5]).localtime
  result
 end

 # reduce 19 omitted

 # reduce 20 omitted

 def _reduce_21( val, _values, result )
 result = (result.to_i * 60 * 60) + (val[2].to_i * 60)
  result
 end

 def _reduce_22( val, _values, result )
 result = (result.to_i * 60 * 60) + (val[2].to_i * 60) + val[4].to_i
  result
 end

 def _reduce_23( val, _values, result )
 result = ::TMail.zonestr2i( val[0] ) * 60
  result
 end

 def _reduce_24( val, _values, result )
        @field.from  = val[0] if val[0]
        @field.by    = val[1] if val[1]
        @field.via   = val[2] if val[2]
        @field.with.replace val[3]
        @field.msgid = val[4] if val[4]
        @field.for_  = val[5] if val[5]
        @field.date  = val[6] if val[6]
  result
 end

 # reduce 25 omitted

 def _reduce_26( val, _values, result )
        result = val[1].join('.')
  result
 end

 def _reduce_27( val, _values, result )
        result = val[3].join('.')
  result
 end

 def _reduce_28( val, _values, result )
        result = val[1].join('.')
  result
 end

 # reduce 29 omitted

 def _reduce_30( val, _values, result )
        result = val[1].join('.')
  result
 end

 # reduce 31 omitted

 def _reduce_32( val, _values, result )
        result = val[1]
  result
 end

 def _reduce_33( val, _values, result )
        result = []
  result
 end

 def _reduce_34( val, _values, result )
        result.push val[2]
  result
 end

 # reduce 35 omitted

 def _reduce_36( val, _values, result )
        result = val[1]
  result
 end

 def _reduce_37( val, _values, result )
        result = val[1]
  result
 end

 # reduce 38 omitted

 def _reduce_39( val, _values, result )
 result = val[1].address
  result
 end

 # reduce 40 omitted

 def _reduce_41( val, _values, result )
 result = val[1]
  result
 end

 # reduce 42 omitted

 def _reduce_43( val, _values, result )
        @field.route.replace result.route
        @field.addr = result.addr
  result
 end

 def _reduce_44( val, _values, result )
 result = val
  result
 end

 def _reduce_45( val, _values, result )
 result.push val[2]
  result
 end

 # reduce 46 omitted

 # reduce 47 omitted

 def _reduce_48( val, _values, result )
 result = val
  result
 end

 def _reduce_49( val, _values, result )
 result.push val[2]
  result
 end

 # reduce 50 omitted

 # reduce 51 omitted

 def _reduce_52( val, _values, result )
        val[1].phrase = HFdecoder.decode( result )
        result = val[1]
  result
 end

 def _reduce_53( val, _values, result )
 result = AddressGroup.new( result, val[2] )
  result
 end

 def _reduce_54( val, _values, result )
        result = val[2]
        result.route = val[1]
  result
 end

 def _reduce_55( val, _values, result )
        result = val[1]
  result
 end

 # reduce 56 omitted

 def _reduce_57( val, _values, result )
 result = [ val[1] ]
  result
 end

 def _reduce_58( val, _values, result )
 result.push val[3]
  result
 end

 def _reduce_59( val, _values, result )
 result = Address.new( val[0], val[2] )
  result
 end

 def _reduce_60( val, _values, result )
 result = Address.new( result, nil )
  result
 end

 def _reduce_61( val, _values, result )
        result = val
  result
 end

 def _reduce_62( val, _values, result )
        result.push val[2]
  result
 end

 def _reduce_63( val, _values, result )
        result = val
  result
 end

 def _reduce_64( val, _values, result )
        result.push val[2]
  result
 end

 # reduce 65 omitted

 # reduce 66 omitted

 # reduce 67 omitted

 def _reduce_68( val, _values, result )
        val[1] = val[1].addr
        result = val.join('')
  result
 end

 # reduce 69 omitted

 def _reduce_70( val, _values, result )
 result << ' ' << val[1]
  result
 end

 # reduce 71 omitted

 # reduce 72 omitted

 # reduce 73 omitted

 # reduce 74 omitted

 # reduce 75 omitted

 def _reduce_76( val, _values, result )
        @field.encrypter = val[0]
  result
 end

 def _reduce_77( val, _values, result )
        @field.encrypter = val[0]
        @field.keyword   = val[1]
  result
 end

 def _reduce_78( val, _values, result )
        @field.major = val[0].to_i
        @field.minor = val[2].to_i
  result
 end

 def _reduce_79( val, _values, result )
        @field.main = val[0]
        @field.sub  = val[2]
  result
 end

 def _reduce_80( val, _values, result )
        @field.main = val[0]
        @field.sub  = ''
  result
 end

 # reduce 81 omitted

 def _reduce_82( val, _values, result )
        @field.params[ val[2].downcase ] = val[4]
  result
 end

 # reduce 83 omitted

 # reduce 84 omitted

 def _reduce_85( val, _values, result )
 @field.encoding = val[0]
  result
 end

 def _reduce_86( val, _values, result )
 @field.disposition = val[0]
  result
 end

 # reduce 87 omitted

 # reduce 88 omitted

 # reduce 89 omitted

 def _reduce_90( val, _values, result )
 @field.params[ val[0].downcase ] = val[2]
  result
 end

 # reduce 91 omitted

 # reduce 92 omitted

 # reduce 93 omitted

 # reduce 94 omitted

 # reduce 95 omitted

 # reduce 96 omitted

 # reduce 97 omitted

end

end   # module TMail

